format zx81
;labelusenumeric
;LISTOFF
	// hardware options to be set and change defaults in ZX81DEF.INC
	MEMAVL	   =	   MEM_16K	   // can be MEM_1K, MEM_2K, MEM_4K, MEM_8K, MEM_16K, MEM_32K, MEM_48K
	STARTMODE  EQU	   SLOW_MODE	   // SLOW or FAST
	DFILETYPE  EQU	   COLLAPSED	    // COLLAPSED or EXPANDED or AUTO
	include '..\..\SINCL-ZX\ZX81.INC'     // definitions of constants
;LISTON

	AUTOLINE 10

	REM _asm
Main:
;init screen handler
	call	EnableHires
;init AY-chip, all registers = 0
	ld	hl,SCREEN_0
	call	UpdateAYRegisters
MainLoop:
	ld	de,FileName
	call	StreamVideoAndAudio
	jp	MainLoop
	ret

;--------------------------
;ffmpeg -i BadApple720p60fps.mp4 -an -vf "eq=contrast=1.05" -s 128x98 -sws_dither ed -pix_fmt monow -f rawvideo -r 25 BadApple_128x98_ed-dither.bin
;--------------------------

include 'LED-macros.INC'

FileName:
	dbzx	'stream.bin;E'		;";E" to get into 512-byte mode
	db	0

AY_HEADER	EQU	32

;de = file path, zero-terminated zx-string
StreamVideoAndAudio:
;prepare zxpand buffer for write
	ld	bc,PRT0
	ld	a,$ff
	out	(c),a
;send filename to zxpand
	ld	bc,PRT2
	.NameOutLoop:
		ld	a,(de)
		out	(c),a		;one character to zxpand
		inc	de
		and	a
	jp	nz,.NameOutLoop 	;repeat until zero-termination
;open file for read
	ld	bc,PRT4
	xor	a
	out	(c),a
	call	GetResponse
;do not read meta-data, not needed

	ld	bc,4800 		;nr of frames to stream
BigLoop:
		push	bc			;save frame counter

;read 3 * 512 bytes from the file
;first 2 * 16 bytes are AY-data
;then 1504 bytes video data, 128x94 resolution
;AY-data is updated every frame @50Hz
;video data is updated every second frame @25Hz
ReadToFrameBuffer0:
		ld	hl,SCREEN_0 + 512 * 0
		call	RequestAndRead512BytesFromFileToHL

		ld	hl,SCREEN_0
		call	UpdateAYRegisters

		ld	hl,SCREEN_0 + 512 * 1
		call	RequestAndRead512BytesFromFileToHL

		ld	hl,SCREEN_0 + 16
		call	UpdateAYRegisters

		ld	hl,SCREEN_0 + 512 * 2
		call	RequestAndRead512BytesFromFileToHL

		call	WaitScreen
ChangeScreenPointer0:
		ld	hl,SCREEN_0 + AY_HEADER
		ld	(ScreenAddress),hl

CheckNumberOfFramesShown0:
		pop	bc			;restore frame counter
		dec	bc
		ld	a,b
		or	c
		jp	z,CloseFile		;continue until all frames shown
		push	bc			;save frame counter

ReadToFrameBuffer1:
		ld	hl,SCREEN_1 + 512 * 0
		call	RequestAndRead512BytesFromFileToHL

		ld	hl,SCREEN_1
		call	UpdateAYRegisters

		ld	hl,SCREEN_1 + 512 * 1
		call	RequestAndRead512BytesFromFileToHL

		ld	hl,SCREEN_1 + 16
		call	UpdateAYRegisters

		ld	hl,SCREEN_1 + 512 * 2
		call	RequestAndRead512BytesFromFileToHL

		call	WaitScreen
ChangeScreenPointer1:
		ld	hl,SCREEN_1 + AY_HEADER
		ld	(ScreenAddress),hl

CheckNumberOfFramesShown1:
		pop	bc			;restore frame counter
		dec	bc
		ld	a,b
		or	c
	jp	nz,BigLoop		;continue until all frames shown

CloseFile:
	ld	bc,PRT4
	ld	a,$80
	out	(c),a
	call	GetResponse
	ret

GetResponse:
.DelayStart:
		in	a,(c)			;get response code
	jp	m,.DelayStart		;and loop while negative
	cp	$40			;all ok
	ret	z			;ret if ok
ErrorHandling:
	jp	$


;Destination: HL
;Destroys: AF, BC, HL
RequestAndRead512BytesFromFileToHL:
;        LED_RED_ON
;        nop
;        nop
;request 512 bytes from file
	ld	bc,PRT5
	ld	a,0
	out	(c),a
;wait for zxpand to finish command
	.DelayStart:
		nop
		in	a,(c)
	jp	m,.DelayStart
;prepare zxpand buffer for read
	nop
	nop
	ld	bc,PRT0
	ld	a,0
	out	(c),a
;a short delay
	repeat 6
		nop
	end repeat
;fast read 512 bytes from zxpand to screen-mem
	repeat 512
		ini
	end repeat
;        LED_RED_OFF
	ret

;update the 16 AY-registers with the data at HL
;Source: HL
;Destroys: AF, BC, HL
UpdateAYRegisters:
	ld	b,16
AYDataLoop:
		ld	a,16
		sub	b
		out	($cf),a 		;set index
;                out     ($df),a                 ;set index
		ld	a,(hl)			;read data
		out	($0f),a 		;write data
;                out     ($1f),a                 ;write data
		inc	hl
	djnz	AYDataLoop
	ret

;sync to screen update
WaitScreen:
	ld	hl,FrameCounter
	ld	a,(hl)
.FrameLoop:
	cp	(hl)
	jp	z,.FrameLoop
	ret

include 'VideoRoutines.inc'
	END _asm
AUTORUN:
	RAND USR #Main

include '..\..\SINCL-ZX\ZX81DISP.INC'	;include D_FILE and needed memory areas

	align	32
SCREEN_0:
	repeat AY_HEADER + SCREEN_WIDTH_BYTES * SCREEN_HEIGHT_RASTERS
		db	0
	end repeat

	align	32
SCREEN_1:
	repeat AY_HEADER + SCREEN_WIDTH_BYTES * SCREEN_HEIGHT_RASTERS
		db	0
	end repeat

VARS_ADDR:
	db 80h				;DO NOT REMOVE!!!

WORKSPACE:

assert ($-MEMST)<MEMAVL
// end of program